@@ -1,17 +1,25 @@
-
 package Shell::Perl;
 
-use 5.006;
 use strict;
 use warnings;
 
-# /Id: Perl.pm 1131 2007-01-27 17:43:35Z me / # don't erase that for now
-# $Id: /iperl/lib/Shell/Perl.pm 2317 2008-03-09T16:22:00.577930Z a.r.ferreira@gmail.com  $
-
-our $VERSION = '0.0022';
+our $VERSION = '0.0026';
 
 use base qw(Class::Accessor); # soon use base qw(Shell::Base);
-Shell::Perl->mk_accessors(qw( out_type dumper context package term ornaments )); # XXX use_strict
+Shell::Perl->mk_accessors(qw(
+    out_type
+    dumper
+    context
+    package
+    perl_version
+    term
+    ornaments
+    library
+)); # XXX use_strict
+
+use lib ();
+use Getopt::Long 2.43 qw(:config no_auto_abbrev no_ignore_case bundling_values);
+use version 0.77;
 
 use Term::ReadLine;
 use Shell::Perl::Dumper;
@@ -24,8 +32,9 @@ use Shell::Perl::Dumper;
 
 sub new {
     my $self = shift;
-    my $sh = $self->SUPER::new({ 
+    my $sh = $self->SUPER::new({
                            context => 'list', # print context
+                           perl_version => $],
                            @_ });
     $sh->_init;
     return $sh;
@@ -53,10 +62,16 @@ sub _init {
         if ($dumper_for{$format}->is_available) {
             #$self->print("format: $format\n");
             $self->set_out($format);
-            last 
+            last
         } # XXX this is not working 100% - and I have no clue about it
     }
 
+    # Set library paths
+    if ($self->library) {
+        warn "Setting library paths (@{$self->library})\n";
+        lib->import(@{ $self->library });
+    }
+
     $self->set_package( __PACKAGE__ . '::sandbox' );
 
 }
@@ -74,7 +89,7 @@ sub print {
 ## # XXX remove: code and docs
 ## sub out {
 ##     my $self = shift;
-## 
+##
 ##     # XXX I want to improve this: preferably with an easy way to add dumpers
 ##     if ($self->context eq 'scalar') {
 ##         $self->print($self->dumper->dump_scalar(shift), "\n");
@@ -85,14 +100,14 @@ sub print {
 
 # XXX I want to improve this: preferably with an easy way to add dumpers
 
-=begin private 
+=begin private
 
 =item B<_print_scalar>
 
     $sh->_print_scalar($answer);
 
 That corresponds to the 'print' in the read-eval-print
-loop (in scalar context). It outputs the evaluation result 
+loop (in scalar context). It outputs the evaluation result
 after passing it through the current dumper.
 
 =end private
@@ -104,14 +119,14 @@ sub _print_scalar { # XXX make public, document
     $self->print($self->dumper->dump_scalar(shift));
 }
 
-=begin private 
+=begin private
 
 =item B<_print_scalar>
 
     $sh->_print_list(@answers);
 
 That corresponds to the 'print' in the read-eval-print
-loop (in list context). It outputs the evaluation result 
+loop (in list context). It outputs the evaluation result
 after passing it through the current dumper.
 
 =end private
@@ -136,7 +151,7 @@ sub set_out {
     if (!defined $dumper_class) {
         $self->_warn("unknown dumper $type");
         return;
-    } 
+    }
     if ($dumper_class->is_available) {
         $self->dumper($dumper_class->new);
         $self->out_type($type);
@@ -184,6 +199,43 @@ sub set_package {
     }
 }
 
+# $err = _check_perl_version($version);
+sub _check_perl_version {
+    my $version = shift;
+    my $ver = eval { version->parse($version) };
+    if ($@) {
+        (my $err = $@) =~ s/at \S+ line \d+.$//;
+        return $err;
+    }
+    # Current perl
+    my $v = $^V || version->parse($]);
+    if ($ver > $v) {
+        return "This is only $v";
+    }
+    return undef; # good
+}
+
+sub set_perl_version {
+    my $self = shift;
+    my $version = shift;
+
+    if (!defined $version) {
+        $self->perl_version($]);
+    }
+    elsif ($version eq q{''} || $version eq q{""}) {
+        $self->perl_version('');
+    }
+    else {
+        my $err = _check_perl_version($version);
+        if ($err) {
+            $self->_warn("bad perl_version ($version): $err");
+        }
+        else {
+            $self->perl_version($version);
+        }
+    }
+}
+
 use constant HELP =>
     <<'HELP';
 Shell commands:           (begin with ':')
@@ -191,6 +243,7 @@ Shell commands:           (begin with ':')
   :set out (D|DD|DDS|Y|P) - setup the output format
   :set ctx (scalar|list|void|s|l|v|$|@|_) - setup the eval context
   :set package <name> - set package in which shell eval statements
+  :set perl_version <version> - set perl version to eval statements
   :reset - reset the environment
   :dump history <file> - (experimental) print the history to STDOUT or a file
   :h(elp) - get this help screen
@@ -230,7 +283,7 @@ sub _readline {
 sub _history_file { # XXX
     require Path::Class;
     require File::HomeDir;
-    return Path::Class::file( File::HomeDir->my_home, '.pirl-history' )->stringify;
+    return Path::Class::file( File::HomeDir->my_home, '.pirl-history-xxx' );
 }
 
 sub _read_history { # XXX belongs to Shell::Perl::ReadLine
@@ -238,12 +291,10 @@ sub _read_history { # XXX belongs to Shell::Perl::ReadLine
     my $h    = _history_file;
     #warn "read history from $h\n"; # XXX
     if ( $term->Features->{readHistory} ) {
-        $term->ReadHistory( $h );
+        $term->ReadHistory( "$h" );
     } elsif ( $term->Features->{setHistory} ) {
         if ( -e $h ) {
-            require File::Slurp;
-            my @h = File::Slurp::read_file( $h );
-            chomp @h;
+            my @h = $h->slurp( chomp => 1 );
             $term->SetHistory( @h );
         }
     } else {
@@ -257,11 +308,10 @@ sub _write_history { # XXX belongs to Shell::Perl::ReadLine
    my $h    = _history_file;
    #warn "write history to $h\n"; # XXX
    if ( $term->Features->{writeHistory} ) {
-       $term->WriteHistory( $h );
+       $term->WriteHistory( "$h" );
    } elsif ( $term->Features->{getHistory} ) {
-       require File::Slurp;
-       my @h = map { "$_\n" } $term->GetHistory;
-       File::Slurp::write_file( $h, @h );
+       my @h = $term->GetHistory;
+       $h->spew_lines(\@h);
    } else {
        # warn "Your ReadLine doesn't support getHistory\n";
    }
@@ -279,7 +329,7 @@ sub run {
     my $self = shift;
     my $shell_name = _shell_name;
     $self->term( my $term = $self->_new_term( $shell_name ) );
-    $term->ornaments($self->ornaments); # XXX 
+    $term->ornaments($self->ornaments); # XXX
     my $prompt = "$shell_name > ";
 
     print "Welcome to the Perl shell. Type ':help' for more information\n\n";
@@ -297,6 +347,7 @@ sub run {
             $self->set_out($1) if /^:set out (\S+)/;
             $self->set_ctx($1) if /^:set ctx (\S+)/;
             $self->set_package($1) if /^:set package (\S+)/;
+            $self->set_perl_version($1) if /^:set perl_version(?: (\S+))?/;
             $self->reset if /^:reset/;
             $self->help if /^:h(elp)?/;
             $self->dump_history($1) if /^:dump history(?:\s+(\S*))?/;
@@ -327,16 +378,31 @@ sub run {
 
 }
 
+sub _package_stmt {
+    my $package = shift->package;
+    ("package $package");
+}
+
+sub _use_perl_stmt {
+    my $perl_version = shift->perl_version;
+    $perl_version ? ("use $perl_version") : ();
+}
+
 # $shell->eval($exp)
 sub eval {
     my $self = shift;
     my $exp = shift;
-    my $package = $self->package;
-    # XXX gotta restore $_, etc.
 
+    my $preamble = join ";\n", (
+        $self->_package_stmt,
+        $self->_use_perl_stmt,
+        "no strict qw(vars subs)",
+        "",    # for the trailing ;
+    );
+
+    # XXX gotta restore $_, etc.
     return eval <<CHUNK;
-       package $package; # XXX
-       no strict qw(vars subs);
+       $preamble
 #line 1
        $exp
 CHUNK
@@ -351,14 +417,14 @@ sub quit {
 }
 
 sub run_with_args {
-    my $self = shift; 
+    my $self = shift;
 
     # XXX do something with @ARGV (Getopt)
     my %options = ( ornaments => 1 );
     if ( @ARGV ) {
         # only require Getopt::Long if there are actually command line arguments
         require Getopt::Long;
-        Getopt::Long::GetOptions( \%options, 'ornaments!', 'version|v' );
+        Getopt::Long::GetOptions( \%options, 'ornaments!', 'version|v', 'library|I=s@' );
     }
 
     my $shell = Shell::Perl->new(%options);
@@ -389,10 +455,10 @@ sub dump_history {
     }
 
     if ( $file ) {
-        open( my $fh, ">>", $file ) 
+        open( my $fh, ">>", $file )
             or do { warn "Couldn't open '$file' for history dump\n"; return; };
-        for ( $self->term->GetHistory ) { 
-            print $fh $_, "\n"; 
+        for ( $self->term->GetHistory ) {
+            print $fh $_, "\n";
         }
         close $fh;
 
@@ -407,7 +473,7 @@ sub dump_history {
 1;
 
 # OUTPUT Data::Dump, Data::Dumper, YAML, others
-# document: use a different package when eval'ing 
+# document: use a different package when eval'ing
 # reset the environment
 # implement shell commands (:quit, :set, :exit, etc.)
 # how to implement array contexts?
@@ -433,14 +499,15 @@ sub dump_history {
 ##
 ##sub out { ? }
 
-# svn:keywords Id
-# svn:eol-style LF
-
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
-Shell::Perl - A read-eval-print loop in Perl 
+Shell::Perl - A read-eval-print loop in Perl
 
 =head1 SYNOPSYS
 
@@ -449,7 +516,7 @@ Shell::Perl - A read-eval-print loop in Perl
 
 =head1 DESCRIPTION
 
-This is the implementation of a command-line interpreter for Perl. 
+This is the implementation of a command-line interpreter for Perl.
 I wrote this because I was tired of using B<irb> when
 needing a calculator with a real language within. Ah,
 that and because it was damn easy to write it.
@@ -552,6 +619,23 @@ of the expression.
     pirl @> $var = 'a 1 2 3'; $var =~ /(\w+) (\d+) (\d+)/ #scalar
     1
 
+=item :set perl_version
+
+Changes the perl version (and current feature bundle)
+used to evaluate each statement. Usage examples are:
+
+    :set perl_version 5.008
+    :set perl_version v5.10
+    :set perl_version        # current perl version, $]
+
+Default is to use the current perl version, which works like C<eval "use $];">.
+
+Set to an empty string, as in
+
+    :set perl_version ''
+
+for the behavior of pirl 0.0023 or earlier.
+
 =item :reset
 
 Resets the environment, erasing the symbols created
@@ -586,7 +670,7 @@ Starts the read-eval-print loop after reading
 options from C<@ARGV>. It is a class method.
 
 If an option B<-v> or B<--version> is provided,
-instead of starting the REPL, it prints 
+instead of starting the REPL, it prints
 the script identification and exits with 0.
 
    $ pirl -v
@@ -598,7 +682,7 @@ the script identification and exits with 0.
 
 The same as C<run_with_args> but with no code for
 interpreting command-line arguments. It is an instance method,
-so that C<Shell::Perl->run_with_args> is kind of:
+so that C<< Shell::Perl->run_with_args >> is kind of:
 
     Shell::Perl->new->run;
 
@@ -608,7 +692,7 @@ so that C<Shell::Perl->run_with_args> is kind of:
     @answer = $sh->eval($exp);
 
 Evaluates the user input given in C<$exp> as Perl code and returns
-the result. That is the 'eval' part of the 
+the result. That is the 'eval' part of the
 read-eval-print loop.
 
 =item B<print>
@@ -616,7 +700,7 @@ read-eval-print loop.
     $sh->print(@args);
 
 Prints a list of args at the output stream currently used
-by the shell. 
+by the shell.
 
 =item B<help>
 
@@ -657,6 +741,12 @@ must be one of C< ( 'scalar', 'list', 'void',
 Changes current evaluation package. Doesn't change if the
 new package name is malformed.
 
+=item B<set_perl_version>
+
+    $sh->set_perl_version($version);
+
+Changes perl version used to evaluate statements.
+
 =item B<set_out>
 
     $sh->set_out($dumper);
@@ -672,7 +762,7 @@ the evaluation results. Actually must be one of
     $prompt = $sh->prompt_title;
 
 Returns the current prompt which changes with
-executable name and context. For example, 
+executable name and context. For example,
 "pirl @>", "pirl $>", and "pirl >".
 
 =item B<quit>
@@ -699,8 +789,8 @@ saves the terminal history file.
 
 =head2 ABOUT EVALUATION
 
-When the statement read is evaluated, this is done 
-at a different package, which is C<Shell::Perl::sandbox> 
+When the statement read is evaluated, this is done
+at a different package, which is C<Shell::Perl::sandbox>
 by default.
 
 So:
@@ -751,26 +841,6 @@ Refactor the code to promote easy customization of features.
 
 =back
 
-=head1 SEE ALSO
-
-This project is hosted at Google Code:
-
-    http://code.google.com/p/iperl/
-
-To know about interactive Perl interpreters, there are two
-FAQS contained in L<perlfaq3> which are good starting points.
-Those are
-
-    How can I use Perl interactively?
-    http://perldoc.perl.org/perlfaq3.html#How-can-I-use-Perl-interactively%3f
-
-    Is there a Perl shell?
-    http://perldoc.perl.org/perlfaq3.html#How-can-I-use-Perl-interactively%3f
-
-An extra list of Perl shells can be found here:
-
-    http://www.focusresearch.com/gregor/document/psh-1.1.html#other_perl_shells
-
 =head1 BUGS
 
 It is a one-line evaluator by now.
@@ -784,18 +854,53 @@ There are some quirks with Term::Readline (at least on Windows).
 
 There are more bugs. I am lazy to collect them all and list them now.
 
-Please report bugs via CPAN RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Shell-Perl>
-or L<mailto://bugs-Shell-Perl@rt.cpan.org>.
+Please report bugs via Github L<https://github.com/aferreira/pirl/issues>.
+
+=head1 SEE ALSO
+
+This project is hosted at Github:
+
+    https://github.com/aferreira/pirl
+
+To know about interactive Perl interpreters, there are two
+FAQS contained in L<perlfaq3> which are good starting points.
+Those are
+
+    How can I use Perl interactively?
+    http://perldoc.perl.org/perlfaq3.html#How-can-I-use-Perl-interactively%3f
+
+    Is there a Perl shell?
+    http://perldoc.perl.org/perlfaq3.html#How-can-I-use-Perl-interactively%3f
+
+Also:
+
+=over 4
+
+=item *
+
+L<Devel::REPL>
+
+=item *
+
+L<Reply>
+
+=item *
+
+L<A comparison of various REPLs|http://shadow.cat/blog/matt-s-trout/mstpan-17/>
+
+=back
 
 =head1 AUTHORS
 
-Adriano R. Ferreira, E<lt>ferreira@cpan.orgE<gt>
+Adriano R. Ferreira, E<lt>ferreiraE<64>cpan.orgE<gt>
 
-Caio Marcelo, E<lt>cmarcelo@gmail.comE<gt>
+Caio Marcelo, E<lt>cmarceloE<64>gmail.comE<gt>
+
+Ron Savage, E<lt>ronE<64>savage.net.auE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007–2011 by Adriano R. Ferreira
+Copyright (C) 2007–2017 by Adriano R. Ferreira
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
